@@ -398,6 +398,7 @@ class CiviCRM_For_WordPress_Basepage {
 
     // Override page title with high priority
     add_filter( 'wp_title', array( $this, 'wp_page_title' ), 100, 3 );
+    add_filter( 'document_title_parts', array( $this, 'wp_page_title_parts' ), 100, 1 );
 
     // Add compatibility with WordPress SEO plugin's Open Graph title
     add_filter( 'wpseo_opengraph_title', array( $this, 'wpseo_page_title' ), 100, 1 );
@@ -470,6 +471,31 @@ class CiviCRM_For_WordPress_Basepage {
   }
 
 
+  /**
+   * Get CiviCRM basepage title for <title> element.
+   *
+   * Callback method for 'document_title_parts' hook. This filter was introduced
+   * in WordPress 3.8 but it depends on whether the theme has implemented that
+   * method for generating the title or not.
+   *
+   * @since 5.14
+   *
+   * @param array $parts The existing title parts.
+   * @return array $parts The modified title parts.
+   */
+  public function wp_page_title_parts( $parts ) {
+
+    // Override with CiviCRM's title
+    if ( isset( $parts['title'] ) ) {
+      $parts['title'] = $this->basepage_title;
+    }
+
+    // Return modified title parts
+    return $parts;
+
+  }
+
+
   /**
    * Get CiviCRM base page title for Open Graph elements.
    *
@@ -529,19 +555,34 @@ class CiviCRM_For_WordPress_Basepage {
    */
   public function basepage_canonical_url( $canonical ) {
 
-    /*
-     * It would be better to specify which params are okay to accept as the
-     * canonical URLs, but this will work for the time being.
-     */
-    if ( empty( $_GET['page'] )
-      || empty( $_GET['q'] )
-      || 'CiviCRM' !== $_GET['page'] ) {
-      return $canonical;
+    // Access Civi config object
+    $config = CRM_Core_Config::singleton();
+
+    // Retain old logic when not using clean URLs
+    if (!$config->cleanURL) {
+
+      /*
+       * It would be better to specify which params are okay to accept as the
+       * canonical URLs, but this will work for the time being.
+       */
+      if ( empty( $_GET['page'] )
+        || empty( $_GET['q'] )
+        || 'CiviCRM' !== $_GET['page'] ) {
+        return $canonical;
+      }
+      $path = $_GET['q'];
+      unset( $_GET['q'] );
+      unset( $_GET['page'] );
+      $query = http_build_query( $_GET );
+
+    }
+    else {
+
+      $argdata = $this->civi->get_request_args();
+      $path = $argdata['argString'];
+      $query = http_build_query( $_GET );
+
     }
-    $path = $_GET['q'];
-    unset( $_GET['q'] );
-    unset( $_GET['page'] );
-    $query = http_build_query( $_GET );
 
     // We should, however, build the URL the way that CiviCRM expects it to be
     // (rather than through some other funny base page).
